@@ -20,11 +20,8 @@
 #
 """Module table provides class Table."""
 
-from asyncio import create_task, wait
 from itertools import chain
 
-from toposort import toposort
-
 from ajsonapi.attribute import Attribute
 from ajsonapi.conversions import pascal_to_snake
 from ajsonapi.exceptions import ModelError
@@ -36,6 +33,7 @@ from ajsonapi.relationships import (
     ManyToOneRelationship,
     Relationship,
 )
+from ajsonapi.toposort import CircularDependencyError, toposort
 from ajsonapi.uri.collection import Collection
 
 
@@ -152,21 +150,36 @@ class Table:
                             break
 
     @classmethod
-    async def _create(cls):
-        """Creates the SQL table, which represents this association table
-        class.
+    def sql_statements(cls, exclude_classes=None):
+        """Creates the sql statements for this table.
 
-        Args:
-            pool: Connection pool to the database in which to create the SQL
-                tables for this class of the object model.
+        Returns:
+            stmt_table: The SQL statement to create the table.
+            stmts_fk: List of SQL statements for foreign key constraints that
+                are removed from the stmt_table.
         """
-        table_fields = ', '.join(
+        if exclude_classes is None:
+            exclude_classes = set()
+        columns_now, columns_later = [], []
+        for col in cls.columns:
+            if col in cls.local_relationships:
+                if col.rtable in exclude_classes:
+                    columns_later.append(col)
+                else:
+                    columns_now.append(col)
+            else:
+                columns_now.append(col)
+        table_fields = ',\n   '.join(
             chain((col.sql() for col in cls.columns),
-                  (col.sql_constraints() for col in cls.constraints)))
-        stmt = f'CREATE TABLE IF NOT EXISTS {cls.name} ({table_fields})'
-        async with cls.pool.acquire() as connection:
-            log.debug(stmt)
-            return await connection.execute(stmt)
+                  (col.sql_constraints()
+                   for col in cls.constraints
+                   if col in columns_now)))
+        stmt_table = (f'CREATE SEQUENCE IF NOT EXISTS seq_{cls.name};\n'
+                      f'CREATE TABLE IF NOT EXISTS {cls.name} (\n   '
+                      f'{table_fields}\n);')
+        stmts_fk = list(f'ALTER TABLE {cls.name} ADD {col.sql_constraints()}'
+                        for col in columns_later)
+        return stmt_table, stmts_fk
 
     @classmethod
     def _dependencies(cls):
@@ -193,7 +206,7 @@ class AssociationTable(Table):
     @classmethod
     def sql_constraints(cls):
         """Returns the constraints for classes derived from AssociationTable."""
-        return (f"UNIQUE "
+        return ("UNIQUE "
                 f"({', '.join(rel.lfkey for rel in cls.local_relationships)})")
 
 
@@ -211,16 +224,47 @@ def parse_model():
         raise ModelError(errors)
 
 
-async def create_all():
-    """Creates the SQL tables for the classes in the object model."""
+async def db_execute(pool, stmt):
+    """Execute statement."""
+
+    async with pool.acquire() as connection:
+        log.debug(stmt)
+        return await connection.execute(stmt)
+
+
+async def create_all(pool):
+    """Creates the SQL tables for the classes in the object model.
+
+    Args:
+        pool: Connection pool to the database in which to create the SQL tables.
+    """
 
     # pylint: disable=protected-access
+    stmts_delayed = []  # Delayed foreign key constraints
     dependencies = {
         table: table._dependencies() for table in Table.by_class_name.values()
     }
-    for tables in toposort(dependencies):
-        tasks = {create_task(table._create()) for table in tables}
-        await wait(tasks)
+    while True:
+        try:
+            for tables in toposort(dependencies):
+                for table in tables:
+                    stmt_table, _ = table.sql_statements()
+                    await db_execute(pool, stmt_table)
+            break
+        except CircularDependencyError as exc:
+            dependencies = exc.data
+            itr = iter(dependencies.items())
+            table, deps = next(itr)
+            stmt_table, stmts_fk = table.sql_statements(deps)
+            await db_execute(pool, stmt_table)
+            stmts_delayed.extend(stmts_fk)
+            dependencies = {
+                item: (dep - {table})
+                for item, dep in dependencies.items()
+                if item != table
+            }
+    for stmt in stmts_delayed:
+        await db_execute(pool, stmt)
 
 
 def init(pool):
